/*
  ==============================================================================

    AdsrComponent.cpp
    Created: 2 Aug 2021 4:28:41pm
    Author:  nweihsma

  ==============================================================================
*/

#include <JuceHeader.h>
#include "AdsrComponent.h"

//==============================================================================
AdsrComponent::AdsrComponent(FinalSamplerAudioProcessor& p) : myAudioProcessor(p)
{
    // In your constructor, you should add any child components, and
    // initialise any special settings that your component needs.
        //set up sliders and labels
    sliderAndLabelSetup(myAudioProcessor.getTreeState(), attackSliderAttachment, attackSlider, attackLabel, "ATTACK");
    sliderAndLabelSetup(myAudioProcessor.getTreeState(), decaySliderAttachment, decaySlider, decayLabel, "DECAY");
    sliderAndLabelSetup(myAudioProcessor.getTreeState(), sustainSliderAttachment, sustainSlider, sustainLabel, "SUSTAIN");
    sliderAndLabelSetup(myAudioProcessor.getTreeState(), releaseSliderAttachment, releaseSlider, releaseLabel, "RELEASE");


}

AdsrComponent::~AdsrComponent()
{
}

void AdsrComponent::paint(juce::Graphics& g)
{

    g.fillAll(juce::Colours::darkgreen.darker(0.6f));

}

void AdsrComponent::resized()
{

    //make slider and label one component

    const float startX = 0.0f;
    const float startY = 0.2f;
    const float sliderProportionalWidth = 0.12f;
    const float sliderProportionalHeight = 0.75f;

    attackSlider.setBoundsRelative(startX, startY, sliderProportionalWidth, sliderProportionalHeight);
    decaySlider.setBoundsRelative(startX + sliderProportionalWidth, startY, sliderProportionalWidth, sliderProportionalHeight);
    sustainSlider.setBoundsRelative(startX + (sliderProportionalWidth * 2), startY, sliderProportionalWidth, sliderProportionalHeight);
    releaseSlider.setBoundsRelative(startX + (sliderProportionalWidth * 3), startY, sliderProportionalWidth, sliderProportionalHeight);

}

void AdsrComponent::sliderAndLabelSetup(juce::AudioProcessorValueTreeState& treeState, std::unique_ptr<juce::AudioProcessorValueTreeState::SliderAttachment>& sliderAttachment,

    juce::Slider& slider, juce::Label& label, const juce::String parameterID) {
    //slider attachments

    //initialize with treestate etc
    sliderAttachment = std::make_unique<juce::AudioProcessorValueTreeState::SliderAttachment>(treeState, parameterID, slider);
    //===================================================================================================

    //slider

    //style sliders
    slider.setSliderStyle(juce::Slider::SliderStyle::RotaryHorizontalVerticalDrag);
    float newStartAngleRadians = juce::MathConstants<float>::pi * 0.0f;
    float newEndAngleRadians = juce::MathConstants<float>::pi * 1.5f;
    slider.setRotaryParameters(newStartAngleRadians, newEndAngleRadians, true);

    //set text boxes
    slider.setTextBoxStyle(juce::Slider::TextBoxBelow, true, 40, 15);

    //add and make visible
    addAndMakeVisible(slider);
    //=================================================================================
    //label
    label.setFont(11.0f);

    //set text
    label.setText(parameterID, juce::dontSendNotification); //no notification because it wont change

    label.setJustificationType(juce::Justification::centredTop);
    //attach to slider
    label.attachToComponent(&slider, false);

    //make visible
    addAndMakeVisible(label);

}
