/*
  ==============================================================================

    This file contains the basic framework code for a JUCE plugin editor.

  ==============================================================================
*/

#include "PluginProcessor.h"
#include "PluginEditor.h"

//==============================================================================
FinalSamplerAudioProcessorEditor::FinalSamplerAudioProcessorEditor(FinalSamplerAudioProcessor& p)
    : AudioProcessorEditor(&p), audioProcessor(p), myWaveformThumbnail(p), myAdsrComponent(p)

{
    //set look and feel to my custom class via a pointer
    juce::LookAndFeel::setDefaultLookAndFeel(&myLookAndFeel);

    //make waveform visible
    addAndMakeVisible(myWaveformThumbnail);

    addAndMakeVisible(myAdsrComponent);

    setSize(600, 400);

    //start timer with 30 HZ interval
    //same as calling startTimer(1000 / timerFrequencyHz).
    startTimerHz(30);
}

FinalSamplerAudioProcessorEditor::~FinalSamplerAudioProcessorEditor()
{
    juce::LookAndFeel::setDefaultLookAndFeel(nullptr);
    stopTimer();
}

//==============================================================================
void FinalSamplerAudioProcessorEditor::paint(juce::Graphics& g)
{
    g.fillAll(juce::Colours::darkgreen.darker(0.6f));
    g.setColour(juce::Colours::whitesmoke);


}

void FinalSamplerAudioProcessorEditor::resized()
{
    //set bounds relative gets new bounds and uses floats as percentages of box height/width 
    //this should make them line up in top left of component

    //set bounds of components
    myWaveformThumbnail.setBoundsRelative(0.0f, 0.0f, 1.0f, 0.65f);
    myAdsrComponent.setBoundsRelative(0.0f, 0.75f, 1.0f, 0.25f);

}

void FinalSamplerAudioProcessorEditor::timerCallback() {
    //call repaint on this component (and all subcomponents)
        //this gets called periodically, but I can stop and start the timer
    repaint();
}
