/*
  ==============================================================================

    This file contains the basic framework code for a JUCE plugin editor.

  ==============================================================================
*/

#pragma once

#include <JuceHeader.h>
#include "PluginProcessor.h"
#include "WaveformThumbnailComponent.h"
#include "AdsrComponent.h"
#include "PluginStyle.h"




//==============================================================================
/**
*/
class FinalSamplerAudioProcessorEditor : public juce::AudioProcessorEditor,
    public juce::Timer //to redraw playhead line. needs pure virtual callback()

{
public:
    FinalSamplerAudioProcessorEditor(FinalSamplerAudioProcessor&);
    ~FinalSamplerAudioProcessorEditor() override;

    //==============================================================================
    void paint(juce::Graphics&) override;
    void resized() override;

    //override the pure virtual funct for the timer class
    void timerCallback() override;


private:


    //declare adsr
    AdsrComponent myAdsrComponent;

    //declare waveformthumbnail
    WaveformThumbnailComponent myWaveformThumbnail;

    FinalSamplerAudioProcessor& audioProcessor;

    PluginStyle myLookAndFeel;


    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(FinalSamplerAudioProcessorEditor)
};

/*for drag and drop:
Thought it should be DragAndDropContainer, but "if all that you need to do is to respond to files being
drag-and-dropped from the operating system onto your component, you don't need any of these classes,
you can do this simply by overriding FileDragAndDropTarget::filesDropped()."

 !needs 2 pure virtual functions
*/