/*
  ==============================================================================

    This file contains the basic framework code for a JUCE plugin processor.

  ==============================================================================
*/

#pragma once

#include <JuceHeader.h>

//==============================================================================
/**
*/
class FinalSamplerAudioProcessor : public juce::AudioProcessor,
    public juce::ValueTree::Listener

{
public:
    //==============================================================================
    FinalSamplerAudioProcessor();
    ~FinalSamplerAudioProcessor() override;
    //==============================================================================
    void prepareToPlay(double sampleRate, int samplesPerBlock) override;
    void releaseResources() override;

#ifndef JucePlugin_PreferredChannelConfigurations
    bool isBusesLayoutSupported(const BusesLayout& layouts) const override;
#endif

    void processBlock(juce::AudioBuffer<float>&, juce::MidiBuffer&) override;

    //==============================================================================
    juce::AudioProcessorEditor* createEditor() override;
    bool hasEditor() const override;

    //==============================================================================
    const juce::String getName() const override;

    bool acceptsMidi() const override;
    bool producesMidi() const override;
    bool isMidiEffect() const override;
    double getTailLengthSeconds() const override;

    //==============================================================================
    int getNumPrograms() override;
    int getCurrentProgram() override;
    void setCurrentProgram(int index) override;
    const juce::String getProgramName(int index) override;
    void changeProgramName(int index, const juce::String& newName) override;

    //==============================================================================
    void getStateInformation(juce::MemoryBlock& destData) override;
    void setStateInformation(const void* data, int sizeInBytes) override;

    //load file function to give to loadfile button onclick
    void loadFile();
    //new loadFile with an argument to check drag and drop list
    void loadFile(const juce::String& filePath);

    //helper functions
    int getNumberOfSamplerSounds() { return mySampler.getNumSounds(); }
    juce::AudioBuffer<float>& getWaveformBuffer() { return waveformBuffer; }

    void updateADSR();
    //getters to minimize access given
    juce::ADSR::Parameters& getAdsrParameters() { return myAdsrParameters; }

    juce::AudioProcessorValueTreeState& getTreeState() { return myTreeState; }

    std::atomic<bool>& getNoteIsBeingPlayed() { return noteIsBeingPlayed; }
    //std::atomic<bool>& getShouldBeUpdated() { return shouldBeUpdated; }
    std::atomic<int>& getSampleCount() { return sampleCount; }

private:
    //make juce synthesiser object
    juce::Synthesiser mySampler;

    //I want a polyphonic synth
    const int numberOfSamplerVoices = 3;


    //declare ADSR parameters object
    juce::ADSR::Parameters myAdsrParameters;


    //create valuetreestate object
    juce::AudioProcessorValueTreeState myTreeState;

    //method to create parameter layout object to pass to>>
    juce::AudioProcessorValueTreeState::ParameterLayout createParameterLayout();

    //buffer for read method call in loadfile
        //I was storing more than one channel, it would be a 2d vector
    juce::AudioBuffer<float> waveformBuffer;

    //setup audioFormatReader & Manager
    juce::AudioFormatReader* myFormatReader = nullptr;
    //reader is ptr, so if we choose another file, the whole thing does not get deleted

    juce::AudioFormatManager myFormatManager;

    //value tree listener method I inherited to get
    void valueTreePropertyChanged(juce::ValueTree& treeWhosePropertyHasChanged, const juce::Identifier& propertyThatHasChanged) override;

    //heavy member var helpers
    std::atomic<bool> shouldBeUpdated = false; //
    std::atomic<bool> noteIsBeingPlayed = false; // 
    std::atomic<int>  sampleCount = 0;
    //using atomic as a way to help avoid race condition 

    //==============================================================================
    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(FinalSamplerAudioProcessor)
};
