

#include <JuceHeader.h>
#include "AdsrComponent.h"

//==============================================================================
AdsrComponent::AdsrComponent(juce::AudioProcessorValueTreeState& myTreeState)
{
    //use argument in contructor to access audioprocessor.treestate


        //set up sliders and labels
    sliderAndLabelSetup(myTreeState, attackSliderAttachment, attackSlider, attackLabel, "ATTACK");
    sliderAndLabelSetup(myTreeState, decaySliderAttachment, decaySlider, decayLabel, "DECAY");
    sliderAndLabelSetup(myTreeState, sustainSliderAttachment, sustainSlider, sustainLabel, "SUSTAIN");
    sliderAndLabelSetup(myTreeState, releaseSliderAttachment, releaseSlider, releaseLabel, "RELEASE");

    attackSlider.setTextValueSuffix(" s");
    decaySlider.setTextValueSuffix(" s");
    releaseSlider.setTextValueSuffix(" s");

}

AdsrComponent::~AdsrComponent()
{
}

void AdsrComponent::paint(juce::Graphics& g)
{
    auto bounds = getLocalBounds().reduced(3);
    bounds.setY(bounds.getY() + 3.0f);
    auto adsrLabelSpace = bounds.removeFromTop(16.0f);


    g.setColour(juce::Colours::white);
    g.setFont(18.0f);
    g.drawText("Amplitude Envelope", adsrLabelSpace, juce::Justification::centred);

    g.drawRoundedRectangle(bounds.getX(), bounds.getY(), bounds.getWidth(), bounds.getHeight(), 4.0f, 2.0f);

   

    g.setColour(juce::Colours::white);
    g.setFont(15.0f);
}

void AdsrComponent::resized()
{
    // This method is where you should set the bounds of any child
    // components that your component contains..

     //var to hold bounds of whole plugin, minus a bit
    const auto bounds = getLocalBounds().reduced(10);
    //var for distance between sliders
    const float sliderPadding = 10.0f;
    //var for spacing slider evenly in box
    const float sliderWidth = (bounds.getWidth() / 4) - sliderPadding;
    const float sliderHeight = (bounds.getHeight() * 0.8) - sliderPadding; //??

    //x,y point for first slider
    const float sliderStartPointX = bounds.getX();
    const float sliderStartPointY = (bounds.getY() +42);//roughly 1/3 way up the screen

    //set bounds of sliders
        //set first slider from start x
    attackSlider.setBounds(sliderStartPointX, sliderStartPointY, sliderWidth, sliderHeight);
    //set other sliders using previous's x position 
    decaySlider.setBounds(attackSlider.getRight() + sliderPadding, sliderStartPointY, sliderWidth, sliderHeight);
    sustainSlider.setBounds(decaySlider.getRight() + sliderPadding, sliderStartPointY, sliderWidth, sliderHeight);
    releaseSlider.setBounds(sustainSlider.getRight() + sliderPadding, sliderStartPointY, sliderWidth, sliderHeight);


}


void AdsrComponent::sliderAndLabelSetup(juce::AudioProcessorValueTreeState& treeState, std::unique_ptr<juce::AudioProcessorValueTreeState::SliderAttachment>& sliderAttachment,
    juce::Slider& slider, juce::Label& label, const juce::String parameterID) {
    //slider attachments

    //initialize with treestate etc
    sliderAttachment = std::make_unique<juce::AudioProcessorValueTreeState::SliderAttachment>(treeState, parameterID, slider);
    //===================================================================================================

    //slider

    //style sliders
    slider.setSliderStyle(juce::Slider::SliderStyle::LinearVertical);

    //set text boxes
    slider.setTextBoxStyle(juce::Slider::TextBoxBelow, true, 50, 25);
    label.setJustificationType(juce::Justification::centred);

    //add and make visible
    addAndMakeVisible(slider);

    slider.setColour(juce::Slider::ColourIds::thumbColourId, juce::Colours::gold.brighter());
    slider.setColour(juce::Slider::ColourIds::textBoxOutlineColourId, juce::Colours::gold);
    slider.setColour(juce::Slider::ColourIds::trackColourId, juce::Colours::gold);
    slider.setColour(juce::Slider::ColourIds::backgroundColourId, juce::Colours::whitesmoke.darker());

    //=================================================================================
    //label

    //set text
    label.setFont(15.0f);
    //attach to slider
    label.attachToComponent(&slider, false);

    //make visible
    addAndMakeVisible(label);

}