

#include "FilterData.h"

void FilterData::prepareToPlay(int numberOfChannels, double sampleRate, int samplesPerBlock) {
    //reset filter
    myStateVariableFilter.reset();

    //set up process spec object with values from pluginprocessor, to pass to filter prepare();
    juce::dsp::ProcessSpec spec;
    spec.maximumBlockSize = samplesPerBlock;
    spec.sampleRate = sampleRate;
    spec.numChannels = numberOfChannels;

    //initialize filter
    myStateVariableFilter.prepare(spec);

    //let process know everything is prepared
    hasBeenPrepared = true;
}

void FilterData::process(juce::AudioBuffer<float>& buffer) {

    //jassert(hasBeenPrepared);
        //audio block is a pointer to the buffer, not a set of memory holding audio data
        //normally when creating a dsp process from scratch, you have to create one for each channel,
        //dsp class and audio block eliminates that complexity
    juce::dsp::AudioBlock<float> audioBlockToProcess{ buffer };

    //turn audio block into a non-replacing process context
    juce::dsp::ProcessContextReplacing<float> contextToProcess{ audioBlockToProcess };
    //contextreplacing() because single block is being used for input/output. 
    //I want to take the audio, process it, and replace what was there with the new data

    //call native juce filter process method and pass it the context just created from audio buffer (via audio block)
    myStateVariableFilter.process(contextToProcess);


}
void FilterData::reset() {
    //call native filter reset method. resets all filter params to natural state
    myStateVariableFilter.reset();
}
void FilterData::updateFilterParameters(const int filterType, const float filterResonance, const float filterCutoffFrequency) {

    //set statevariablefilter type
    switch (filterType)
    {
    case 0:
        myStateVariableFilter.setType(juce::dsp::StateVariableTPTFilterType::lowpass);
        break;
    case 1:
        myStateVariableFilter.setType(juce::dsp::StateVariableTPTFilterType::bandpass);
        //this tpt class' bandpass gain can be higher than 0 db, but can be set to be "classic"
       // 0db bandpass by multpliying the result by R2 (would require getting deeper into dsp)
        break;
    case 2:
        myStateVariableFilter.setType(juce::dsp::StateVariableTPTFilterType::highpass);
        break;
    default:
        break;
    }

    //set filter params
    myStateVariableFilter.setCutoffFrequency(filterCutoffFrequency);

    myStateVariableFilter.setResonance(filterResonance);
    //lowpass filter resonance emphasizes or suppresses signals below the cutoff frequency.

    //highpass filter resonance emphasizes or suppresses signals above the cutoff frequency.

    //bandpass filter resonance emphasizes or suppresses the portions of the signal—the frequency band—that surround the defined frequency, set with the Cutoff Frequency parameter.
}