

#pragma once

#include <JuceHeader.h>

//==============================================================================
/*
*/
class OscillatorComponent : public juce::Component
{
public:
    OscillatorComponent(juce::AudioProcessorValueTreeState& treeState, juce::String selectorID, juce::String fmDepthParameterID,
        juce::String fmFrequencyParameterID);
    ~OscillatorComponent() override;

    void paint(juce::Graphics&) override;
    void resized() override;

private:
    //combobox is a dropdown list user can select from
    //declare boxes, labels, and attachments
    juce::ComboBox mainOscillatorWaveSelector;
    std::unique_ptr<juce::AudioProcessorValueTreeState::ComboBoxAttachment> mainWaveSelectorAttachment;

    juce::ComboBox fmOscillatorWaveSelector;
    std::unique_ptr<juce::AudioProcessorValueTreeState::ComboBoxAttachment> fmWaveSelectorAttachment;

    juce::Label mainWaveSelectorLabel{ "Primary Oscillator Wave", "Main Wave Type: " };
    juce::Label fmWaveSelectorLabel{ "FM Oscillator Wave", "FM Wave Type: " };



    //declare sliders, labels, and sliderAttachments for pitch and gain
    juce::Slider gainSlider;
    juce::Slider pitchSlider;

    juce::Label gainLabel {"Oscillator Gain", "Oscillator Gain"};
    juce::Label pitchLabel {"Oscillator Pitch Shift", "Pitch Shifter"}; //have to initialize here to be able to set text without another constructor argument

    std::unique_ptr<juce::AudioProcessorValueTreeState::SliderAttachment> gainSliderAttachment;
    std::unique_ptr<juce::AudioProcessorValueTreeState::SliderAttachment> pitchSliderAttachment;
    
    //declare sliders, labels, and sliderAttachments for mod depth and frequency
    juce::Slider fmDepthSlider;
    juce::Slider fmFrequencySlider;
    
    juce::Label fmDepthLabel {"Modulation Depth", "Modulation Depth"};
    juce::Label fmFrequencyLabel {"Modulatiion Frequency", "Modulation Frequency"}; 

    std::unique_ptr<juce::AudioProcessorValueTreeState::SliderAttachment> fmDepthSliderAttachment;
    std::unique_ptr<juce::AudioProcessorValueTreeState::SliderAttachment> fmFrequencySliderAttachment;

    //helper method
    void sliderAndLabelSetup(juce::AudioProcessorValueTreeState& treeState,
        std::unique_ptr<juce::AudioProcessorValueTreeState::SliderAttachment>& sliderAttachment,
        juce::Slider& slider, juce::Label& label, const juce::String parameterID);

    


    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(OscillatorComponent)
};
