
#include "OscillatorData.h"
void OscillatorData::setOscillatorWaveType(const int oscillatorTypeChoice) {

    //switch statement more efficient than if
    switch (oscillatorTypeChoice)
    {
        //using lambda functions from juce tutorial
    case 0:
        //sine
        initialise([](float input) {return std::sin(input); });
        //this object is a juce oscillator object so i can call it's initialise()
        break;

    case 1:
        //square
        initialise([](float input) {return input < 0.0f ? -1.0f : 1.0f; });
        break;
    case 2:
        //saw
        //using juce cast of pi as float
        initialise([](float input) {return input / juce::MathConstants<float>::pi; });
        break;
    default:
        //juce jassert to pause plugin execution
        jassertfalse;
        break;
    }
}

void OscillatorData::prepareToPlay(juce::dsp::ProcessSpec& mySpec) {
    //setup frequency mod oscillator 
    frequencyModulationOscillator.prepare(mySpec);

    prepare(mySpec);
}

void OscillatorData::getNextAudioBlock(juce::dsp::AudioBlock<float>& myBlock) {
    //using bufferSize processing in main (chunk by chunk), need to find value of fm wave at one exact moment for modulation
    //sample by sample processing for modulating


   //iterate through channels
    for (int channelNumber = 0; channelNumber < myBlock.getNumChannels(); channelNumber++) {
        //iterate through samples in channel
        for (int sampleNumber = 0; sampleNumber < myBlock.getNumSamples(); sampleNumber++) {
            //call oscilators processSample            
            frequencyModulation = frequencyModulationOscillator.processSample(myBlock.getSample(channelNumber, sampleNumber)) * myModulationDepth;
            //takes a float and our block is a vector of floats. multiply modulation depth for gain of fm oscillator
        }
    }

    //process with main oscillator. context replacing because input and output are same block. replacing data
    process(juce::dsp::ProcessContextReplacing<float>(myBlock));
}

void OscillatorData::setWaveFrequency(const int midiNoteNumber) {
 //get value of freqMod oscillator before setting main osc's freq

 //set osc base freq after converting it to hertz and modulating
  //modulation float will be between 0 and 1.
    setFrequency(juce::MidiMessage::getMidiNoteInHertz(midiNoteNumber) + frequencyModulation);

    currentMidiNoteNumber = midiNoteNumber;
}

void OscillatorData::setFrequencyModulationParameters(const float modulationFrequency, const float modulationDepth) {

    frequencyModulationOscillator.setFrequency(modulationFrequency);

    //no setgain() for fmoscillator to set mod depth. need a workaround
     // member var to pass responsibility to where sample processing happens.
    myModulationDepth = modulationDepth;

    //call set frequency again after changing mod params, because setWaveFrequency only gets when new key pressed
     //still need midi note number--> another member var
    auto currentFrequency = juce::MidiMessage::getMidiNoteInHertz(currentMidiNoteNumber) + frequencyModulation;
    if (currentFrequency < 0) setFrequency(currentFrequency * -1.0f);
    else setFrequency(currentFrequency);

    //resetting when negative to avoid issues with modulating out of range
         //before you would hear clicking and lost a few notes on the high end of keys
}


void OscillatorData::setFMOscillatorWaveType(const int oscillatorTypeChoice) {

    //switch statement more efficient than if
    switch (oscillatorTypeChoice)
    {
        //using lambda functions from juce tutorial
    case 0: 
        //sine
        frequencyModulationOscillator.initialise([](float input) {return std::sin(input); });
        //this object is a juce oscillator object so i can call it's initialise()
        break;

    case 1:
        //square
        frequencyModulationOscillator.initialise([](float input) {return input < 0.0f ? -1.0f : 1.0f; });
        break;
    case 2:
        //saw
        //using juce cast of pi as float
        frequencyModulationOscillator.initialise([](float input) {return input / juce::MathConstants<float>::pi; });
        break;
    default:
        //juce jassert to pause plugin execution
        jassertfalse;
        break;
    }
}