/*
  ==============================================================================

    This file contains the basic framework code for a JUCE plugin editor.

  ==============================================================================
*/

#include "PluginProcessor.h"
#include "PluginEditor.h"

//==============================================================================
LastSynthesizerAudioProcessorEditor::LastSynthesizerAudioProcessorEditor(LastSynthesizerAudioProcessor& p)
    : AudioProcessorEditor(&p), audioProcessor(p), myOscillatorComponent(audioProcessor.myTreeState, "WAVETYPE", "MODDEPTH", "MODFREQ"),
    myAdsrComponent(audioProcessor.myTreeState), myFilterComponent("Filter 1", audioProcessor.myTreeState, "FILTERTYPE", "FILTERRESONANCE", "FILTERCUTOFF"),
    mySecondFilterComponent("Filter 2", audioProcessor.myTreeState, "FILTER2TYPE", "FILTER2RESONANCE", "FILTER2CUTOFF")
{
    //set look and feel to my custom class via a pointer
    juce::LookAndFeel::setDefaultLookAndFeel(&myLookAndFeel);

    //set initial size of plugin when opening
    setSize(700, 500);

    //make all my components visible on main editor canvas
    addAndMakeVisible(myOscillatorComponent);
    addAndMakeVisible(myAdsrComponent);
    addAndMakeVisible(myFilterComponent);
    addAndMakeVisible(mySecondFilterComponent);

}
LastSynthesizerAudioProcessorEditor::~LastSynthesizerAudioProcessorEditor()
{
    //reset look and feel on destruct to avoid issues
    juce::LookAndFeel::setDefaultLookAndFeel(nullptr);
}

//==============================================================================
void LastSynthesizerAudioProcessorEditor::paint(juce::Graphics& g)
{
    g.fillAll(juce::Colours::darkgreen.darker());
}

void LastSynthesizerAudioProcessorEditor::resized()
{    
    // how components behave once plugin is resized
    
    //set bounds on whole plugin of components. Everything within these components is already drawn,scaled, and placed
    myOscillatorComponent.setBounds(0, 0, getWidth() / 2, (getHeight()/3) * 2);
    myAdsrComponent.setBounds(getWidth() / 2, 0, getWidth() / 2, (getHeight() / 3) * 2);
    mySecondFilterComponent.setBounds(getWidth() /2, (getHeight() / 3) *2, getWidth() / 2, getHeight() / 3);
    myFilterComponent.setBounds(0, (getHeight() / 3) * 2, getWidth() / 2, getHeight() / 3);

}


