/*
  ==============================================================================

    This file contains the basic framework code for a JUCE plugin editor.

  ==============================================================================
*/

#pragma once

#include <JuceHeader.h>
#include "PluginProcessor.h"
#include "AdsrComponent.h"
#include "OscillatorComponent.h"
#include "FilterComponent.h"
#include "PluginStyle.h"

//==============================================================================
/**
*/
class LastSynthesizerAudioProcessorEditor : public juce::AudioProcessorEditor
{
public:
    LastSynthesizerAudioProcessorEditor(LastSynthesizerAudioProcessor&);
    ~LastSynthesizerAudioProcessorEditor() override;

    void paint(juce::Graphics&) override;
    void resized() override;

private:
    // This reference is provided as a quick way for your editor to
    // access the processor object that created it.
    LastSynthesizerAudioProcessor& audioProcessor;
    //^declared before adsr because adsr needs reference to processor


    //declare oscillator for selecting wave
    OscillatorComponent myOscillatorComponent;
    //^before adsr because adsr effects oscillator

    //declare adsr
    AdsrComponent myAdsrComponent;

    //declare filter 
    FilterComponent myFilterComponent;
    //declare filter 
    FilterComponent mySecondFilterComponent;

    //declare PluginStyle which inherits from juce LookAndFeelV4 to apply slider and combobox customization
    PluginStyle myLookAndFeel;


    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(LastSynthesizerAudioProcessorEditor)
};
