
#include "PluginStyle.h"



//to customize, I tweaked the existing juce code for drawing a rotary slider in "juce_LookAndFeel_V4.cpp"
void PluginStyle::drawRotarySlider(juce::Graphics& g, int x, int y, int width, int height, float sliderPosProportional, float rotaryStartAngle, float rotaryEndAngle, juce::Slider& slider) {

    //local vars to make drawing components easier
    juce::Colour outlineColor = juce::Colours::whitesmoke.darker();
    juce::Colour fillColor = juce::Colours::gold;
    juce::Colour thumbColor = juce::Colours::gold.brighter();


    auto bounds = juce::Rectangle<int>(x, y, width, height).toFloat().reduced(10);

    auto radius = juce::jmin(bounds.getWidth(), bounds.getHeight()) / 2.0f;
    auto toAngle = rotaryStartAngle + sliderPosProportional * (rotaryEndAngle - rotaryStartAngle);
    auto lineWidth = juce::jmin(8.0f, radius * 0.5f);
    auto arcRadius = radius - lineWidth * 0.5f;

    //background arc is portion of dial not shaded (slider hasnt gotten there yet)
        //set arc area using juce path
    juce::Path backgroundArc;
    backgroundArc.addCentredArc(bounds.getCentreX(),
        bounds.getCentreY(),
        arcRadius,
        arcRadius,
        0.0f,
        rotaryStartAngle,
        rotaryEndAngle,
        true);

    g.setColour(outlineColor);
    //draw this path
    g.strokePath(backgroundArc, juce::PathStrokeType(lineWidth, juce::PathStrokeType::curved, juce::PathStrokeType::rounded));

    if (slider.isEnabled())
    {
        //value arc is shaded region of dial track. (amount dial has been turned)
            //set arc area using juce path
        juce::Path valueArc;
        valueArc.addCentredArc(bounds.getCentreX(),
            bounds.getCentreY(),
            arcRadius,
            arcRadius,
            0.0f,
            rotaryStartAngle,
            toAngle,
            true);

        g.setColour(fillColor);
        g.strokePath(valueArc, juce::PathStrokeType(lineWidth, juce::PathStrokeType::curved, juce::PathStrokeType::rounded));
    }
    //thumb is dial head that indicates current position on dial


    auto thumbPointX = (bounds.getCentreX() + arcRadius * std::cos(toAngle - juce::MathConstants<float>::halfPi));
    auto thumbPointY = (bounds.getCentreY() + arcRadius * std::sin(toAngle - juce::MathConstants<float>::halfPi));
    //juce point just holds an X and Y value in one data structure
    juce::Point<float> thumbPoint(thumbPointX, thumbPointY);

    //set colour to gold before drawing line
    g.setColour(thumbColor);
    //draw the thumbpoint (i want a line from thumbpoint to center, not just small dot)
    float dialLineStartX = thumbPoint.getX();
    float dialLineStartY = thumbPoint.getY();
    float dialLineEndX = backgroundArc.getBounds().getCentreX() - 9.0f;
    float dialLineEndY = backgroundArc.getBounds().getCentreY() - 9.0f;

    //draw thumbpoint line with same width as arc line 
    g.drawLine(dialLineStartX, dialLineStartY, dialLineEndX, dialLineEndY, 3.0f);

   
    slider.setColour(juce::Slider::ColourIds::textBoxOutlineColourId, juce::Colours::gold);

}

void PluginStyle::drawComboBox(juce::Graphics& g, int width, int height, bool,
    int, int, int, int, juce::ComboBox& box)
{
    auto cornerSize = box.findParentComponentOfClass<juce::ChoicePropertyComponent>() != nullptr ? 0.0f : 3.0f;
    juce::Rectangle<int> boxBounds(0, 0, width, height);

    g.setColour(juce::Colours::darkgreen.darker());
    g.fillRoundedRectangle(boxBounds.toFloat(), cornerSize);

    g.setColour(juce::Colours::gold);
    g.drawRoundedRectangle(boxBounds.toFloat().reduced(0.5f, 0.5f), cornerSize, 1.0f);

    juce::Rectangle<int> arrowZone(width - 40, 0, 30, height);
    juce::Path path;
    path.startNewSubPath((float)arrowZone.getX() + 3.0f, (float)arrowZone.getCentreY() - 2.0f);
    path.lineTo((float)arrowZone.getCentreX(), (float)arrowZone.getCentreY() + 3.0f);
    path.lineTo((float)arrowZone.getRight() - 3.0f, (float)arrowZone.getCentreY() - 2.0f);
    path.lineTo(((float)arrowZone.getX() + 3.0f), ((float)arrowZone.getCentreY() - 2.0f));


    g.setColour(box.findColour(juce::ComboBox::arrowColourId).withAlpha((box.isEnabled() ? 0.9f : 0.2f)));
    g.strokePath(path, juce::PathStrokeType(2.0f));
}

