
#pragma once
#include <JuceHeader.h>
#include "AdsrData.h"
#include "SynthSound.h"
#include "OscillatorData.h"



class SynthVoice : public juce::SynthesiserVoice ////abstract class for virtual functions
{
public:
    bool canPlaySound(juce::SynthesiserSound* sound) override;

    void startNote(int midiNoteNumber, float velocity, juce::SynthesiserSound* sound, int currentPitchWheelPosition) override;

    void stopNote(float velocity, bool allowTailOff) override;

    void pitchWheelMoved(int newPitchWheelValue) override;

    void controllerMoved(int controllerNumber, int newControllerValue) override;

  
    void prepareToPlay(float pitchValue, float gainValue, double sampleRate, int samplesPerBlock, int outputChannels);

    void renderNextBlock(juce::AudioBuffer< float >& outputBuffer, int startSample, int numSamples) override;
    
    //helper methods to update parameters because i cant pass in extra info to the inherited pure virtual functs
    void updateGain(const float gainValue);
    void updateADSR(const float attackValue, const float decayValue, const float sustainValue, const float releaseValue);
    void updatePitch(const float pitchValue);

    //to give processor access to object
    OscillatorData& getOscillator() { return myOscillatorObject; }

private:
    //make my objects
    AdsrData myAdsrObject;
    OscillatorData myOscillatorObject;

    //for click
    juce::AudioBuffer<float> clickFixBuffer;
    
    //to make sure prepare to play has been called before trying to render
    bool hasBeenPrepared = false;

    //declare gain object
    juce::dsp::Gain<float> myGain;

    //declare value to store altered pitch
    float voiceNewPitchValue = 0.0;
  


};
//FOR SLIDERS:
//synth voic juce::adsr, plugin processor valuetreestate (actual parameters set), editor sliders all need to be linked for functionality 
//of parameter control
//updateADSR() method called in processor processblock() to update adsr with paramter values from valuetreestate. needs to be public